/**
 * Apeman password context.
 * @memberof module:pon-password/lib
 * @constructor Apemanpasswd
 * @param {object} config - Password configuration.
 * @param {string} [config.algorithm='pdkdf2'] - Name of algorithm.
 * @param {number} [config.iterations=100] - Iteration count.
 * @param {number} [config.length=84] - Key length.
 * @param {string} [config.format='base64'] - Digest format.
 */

"use strict";

const extend = require('extend'),
    newSalt = require('./salting/new_salt'),
    pdkdf2Digest = require('./digesting/pdkdf2_digest');

/** @lends Apemanpasswd */
function Apemanpasswd(config) {
    let s = this;
    extend(s, config);
}

Apemanpasswd.prototype = {
    algorithm: 'pdkdf2',
    iterations: 100,
    length: 84,
    format: 'base64',
    /**
     * Digest a password.
     * @param {string} text - Text to digest.
     * @param {string} salt - Salt for digest.
     * @param {function} callback - Callback when done.
     */
    digest: function (text, salt, callback) {
        let s = this;
        switch (s.algorithm) {
            case 'pdkdf2':
                pdkdf2Digest(text, salt, {
                    iterations: s.iterations,
                    length: s.length,
                    format: s.format
                }, callback);
                break;
            default:
                callback(new Error('Unknown algorithm: ' + s.algorithm));
        }
    },
    /**
     * Digest a password.
     * @param {string} text - Text to digest.
     * @param {string} salt - Salt for digest.
     */
    digestSync: function (text, salt) {
        let s = this;
        switch (s.algorithm) {
            case 'pdkdf2':
                return pdkdf2Digest.sync(text, salt, {
                    iterations: s.iterations,
                    length: s.length,
                    format: s.format
                });
        }
    },
    /**
     * Generate a new salt.
     * @param {number} [len=8] - Salt length.
     * @returns {string} - New salt.
     */
    newSalt: function (len) {
        return newSalt(len || 8);
    },
    /**
     * Get spec string.
     * @returns {string} spec - Spec of this password.
     */
    toSpecString: function () {
        let s = this;
        return [s.algorithm, s.iterations, s.length].join(':');
    }
};

module.exports = Apemanpasswd;