/**
 * Create pdkdf2 digest.
 * @memberof module:apemanpasswd/lib/salting
 * @function pdkdf2Digest
 * @param {string} text - Text to digest.
 * @param {string} salt - Digest salt.
 * @param {object} [options] - Optional settings.
 * @param {number} [options.iterations=100] - Iteration count.
 * @param {number} [options.length=84] - Key length.
 * @param {string} [options.format='base64']
 * @param {function} callback - Callback when done.
 */

"use strict";

const crypto = require('crypto'),
    async = require('async'),
    argx = require('argx');

/** @lends pdkdf2Digest */
function pdkdf2Digest(text, salt, options, callback) {
    let args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    if (!salt) {
        throw new Error('Salt is required.');
    }
    let iterations = options.iterations || 100,
        length = options.length || 84,
        format = options.format || 'base64';
    async.waterfall([
        (callback) => {
            crypto.pbkdf2(text, salt, iterations, length, callback);
        },
        (buffer, callback) => {
            var result = Buffer(buffer, 'binary').toString(format);
            callback(null, result);
        }
    ], callback);
}

pdkdf2Digest.sync = function (text, salt, options) {
    let args = argx(arguments);
    options = args.pop('object') || {};
    if (!salt) {
        throw new Error('Salt is required.');
    }
    let iterations = options.iterations || 100,
        length = options.length || 84,
        format = options.format || 'base64';
    let buffer = crypto.pbkdf2Sync(text, salt, iterations, length);
    return Buffer(buffer, 'binary').toString(format);
};

module.exports = pdkdf2Digest;